@@ -29,6 +29,6 @@
 
 __author__ = "Richard Gast"
 __status__ = "Development"
-__version__ = "0.6.3"
+__version__ = "0.7.0"
 
-from .pycobi import ODESystem
+from .pycobi import ODESystem, get_from_solutions, continue_period_doubling_bf
